/*********************************************************************
 *
 *      PIC32MX  Timer1 API Example Code
 *
 *********************************************************************
 * FileName:        core_timer_int.c
 * Dependencies:	plib.h, attribs.h
 *
 * Processor:       PIC32MX
 *
 * Complier:        MPLAB C32
 *                  MPLAB IDE
 * Company:         Microchip Technology Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company? for its PIC32MX Microcontroller is intended
 * and supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip Microcontroller products.
 * The software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS?CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *********************************************************************
 * $Id: core_timer_int.c 9390 2008-06-16 23:43:04Z rajbhartin $
 *
 * $Name:  $
 *********************************************************************
 * The purpose of this example code is to demonstrate the use of
 * peripheral library macros and functions supporting the PIC32MX
 * Core Timer.
 *
 * Platform: 	PIC32MX Starter Kit
 *
 * Features demonstrated:
 *      - Core Timer configuration
 *      - Interrupt configuration
 *      - ISR syntax
 *
 * Description:
 *		- This example generates a core timer interrupt ten times a second
 *
 * Oscillator Configuration Bit Settings:
 * 		- Oscillator Selection Bits = 	Primary Osc w/PLL (XT+HS+EC+PLL)
 *		- Primary Oscillator Config = 	HS osc mode
 *		- PLL Input Divider			=	2x Divider
 *		- PLL Multiplier			=	18x Multiplier
 *
 * Notes:
 *		- Core timer clock frequency is FOSC/2
 *		- CORE_TICK_RATE			= FOSC/2/TOGGLES_PER_SEC
 *									= 72E6/2/100 = 360000
 *		- To toggle led 10 times every second, core compare register
 *		is loaded with 360000.
 *
 ********************************************************************/
#include <plib.h>

// Configuration Bit settings
// SYSCLK = 80 MHz (8MHz Crystal/ FPLLIDIV * FPLLMUL / FPLLODIV)
// PBCLK =  10 MHz
// FRC Osc w/PLL  
// WDT OFF
// Other options are don't care
//
#pragma config FPLLMUL = MUL_15, FPLLIDIV = DIV_2, FPLLODIV = DIV_1, FWDTEN = OFF
#pragma config POSCMOD = HS, FNOSC = FRCPLL, FPBDIV = DIV_1
#pragma config ICESEL =  ICS_PGx2


// Let compile time pre-processor calculate the CORE_TICK_PERIOD
#define SYS_FREQ 				(60000000L)
#define PBCLK 					(60000000L)
#define TOGGLES_PER_SEC			10
#define CORE_TICK_RATE	       (SYS_FREQ/2/TOGGLES_PER_SEC)
#define	T1_TICK_TIME_MS			100

#define	SWITCH_2				PORTDbits.RD6
#define	SWITCH_3				PORTDbits.RD7

void Init_Timer1(void);

unsigned Timer1_Tick_Flag = 0 ;
unsigned UpdateIndex = 0 ;
unsigned T1_TickTime_ms ;

int main(void)
{
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //STEP 1. Configure cache, wait states and peripheral bus clock
	// Configure the device for maximum performance but do not change the PBDIV
	// Given the options, this function will change the flash wait states, RAM
	// wait state and enable prefetch cache but will not change the PBDIV.
	// The PBDIV value is already set via the pragma FPBDIV option above..
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    // Step 2. enable multi-vector interrupts
    INTEnableSystemMultiVectoredInt();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // STEP 3. configure the Timer1
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 	Init_Timer1();

	// *************************************
	// Clear all output first !!
	// LED1 = RB2
	// LED2 = RB3
	// LED3 = RF0
	// LED4 = RF1
	// *************************************
	mPORTBClearBits(BIT_2|BIT_3);	
	mPORTFClearBits(BIT_0|BIT_1);
	
	// *************************************
	// RB2 , RB3 , RF0 , RF1 as output ....
	// *************************************
    mPORTBSetPinsDigitalOut(BIT_2 | BIT_3);
	mPORTFSetPinsDigitalOut(BIT_0 | BIT_1);

	T1_TickTime_ms = T1_TICK_TIME_MS ;		//	Set Initial value !!

    while(1)
	{
		if ( Timer1_Tick_Flag )
		{
			Timer1_Tick_Flag = 0 ;
			switch ( UpdateIndex )
			{
				case 0:	//	Turn off all LEDs then turn LED1 ON 
						mPORTBClearBits(BIT_2|BIT_3);	
						mPORTFClearBits(BIT_0|BIT_1);
						mPORTBSetBits(BIT_2);
						UpdateIndex ++ ;
						break ;
				case 1:	//	Turn off all LEDs then turn LED2 ON
						mPORTBClearBits(BIT_2|BIT_3);	
						mPORTFClearBits(BIT_0|BIT_1);
						mPORTBSetBits(BIT_3);
						UpdateIndex ++ ;
						break ;
				case 2:	//	Turn off all LEDs then turn LED3 ON
						mPORTBClearBits(BIT_2|BIT_3);	
						mPORTFClearBits(BIT_0|BIT_1);
						mPORTFSetBits(BIT_0);
						UpdateIndex ++ ;
						break ;
				case 3:	//	Turn off all LEDs then turn LED4 ON
						mPORTBClearBits(BIT_2|BIT_3);	
						mPORTFClearBits(BIT_0|BIT_1);
						mPORTFSetBits(BIT_1);
						UpdateIndex = 0 ;
						break ;
			}

			if ( (! SWITCH_2) && (T1_TickTime_ms > 50 ))
			{
				T1_TickTime_ms -=2 ;	
				PR1 = (PBCLK/(1000*256))*T1_TickTime_ms-1 ;	
			}
			else if ( (! SWITCH_3) && (T1_TickTime_ms < 200 ))
			{
				T1_TickTime_ms +=2 ;
				PR1 = (PBCLK/(1000*256))*T1_TickTime_ms-1 ;
			}
		}
	}

}

//******************************************************************//
//TIMER1lưƵ{//
//******************************************************************//
void Init_Timer1(void)
{
ConfigIntTimer1( T1_INT_PRIOR_4 | T1_INT_ON ) ;		// NTimer1_uv]w4
													// NTimer1_Pॴ}															
OpenTimer1( T1_ON | T1_IDLE_STOP | T1_GATE_OFF | 	// NTimer1] Gated Timer off  mode !!
			T1_PS_1_256 | T1_SOURCE_INT,((PBCLK/(1000*256))*T1_TICK_TIME_MS-1 )) ;
}

//******************************************************************//
//TIMER1_Ƶ{//
//******************************************************************//
void __ISR(_TIMER_1_VECTOR, ipl4) Timer1Handler(void)
{
		// M Timer1 interrupt Flag ..
		mT1ClearIntFlag() ;
		
		Timer1_Tick_Flag = 1 ;
    	
}
